import static javax.jms.Session.AUTO_ACKNOWLEDGE;

import java.util.HashMap;
import java.util.Map;

import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSConsumer;
import javax.jms.JMSContext;
import javax.jms.JMSProducer;

import org.jboss.logging.Logger;
import org.junit.Before;
import org.junit.Test;

public class ArtemisHornetQClientTest {

    private static Logger _logger = Logger.getLogger(ArtemisHornetQClientTest.class);
    private static final String USER = "guest";
    private static final String PASSWD = "guestp.1";
    private static final String QUEUE = "DLQ";
    private static final int NUM = 5;
    private long start;

    @Before
    public void before() {
        start = System.currentTimeMillis();
    }

    public void sendAndReceiveMessages(ConnectionFactory cf, Destination destination) {
        try (
                JMSContext context = cf.createContext(USER, PASSWD, AUTO_ACKNOWLEDGE)
        ) {
            JMSProducer producer = context.createProducer();
            for (int i=0; i<NUM; i++) {
                String text = "foobar-" + i;
                producer.send(destination, text);
                print(String.format("* * *  Sent a message '%s' to '%s'", text, destination.toString()));
            }
        }
        try (
                JMSContext context = cf.createContext(USER, PASSWD, AUTO_ACKNOWLEDGE)
        ) {
            JMSConsumer consumer = context.createConsumer(destination);
            int count = 0;
            while (count < NUM) {
                String text = consumer.receiveBody(String.class, 1000);
                if (text != null) {
                    count++;
                    print(String.format("* * *  Received a message '%s' to '%s'", text, destination.toString()));
                } else {
                    print(String.format("* * *  No message was received from '%s'", destination.toString()));
                }
            }
        }
    }

    @Test
    public void testHornetQClient() throws Exception {
        Map<String, Object> transportParams = new HashMap<String, Object>();
        transportParams.put(org.hornetq.core.remoting.impl.netty.TransportConstants.HTTP_UPGRADE_ENABLED_PROP_NAME, true);
        transportParams.put(org.hornetq.core.remoting.impl.netty.TransportConstants.HTTP_UPGRADE_ENDPOINT_PROP_NAME, "http-acceptor");
        transportParams.put(org.hornetq.core.remoting.impl.netty.TransportConstants.PORT_PROP_NAME, 8080);
        org.hornetq.api.core.TransportConfiguration transportConfiguration = new org.hornetq.api.core.TransportConfiguration(org.hornetq.core.remoting.impl.netty.NettyConnectorFactory.class.getName(), transportParams);

        print("Starting to send/receive using HornetQ client API");
        ConnectionFactory cf = org.hornetq.api.jms.HornetQJMSClient.createConnectionFactoryWithoutHA(org.hornetq.api.jms.JMSFactoryType.CF, transportConfiguration);
        Destination destination = org.hornetq.api.jms.HornetQJMSClient.createQueue(QUEUE);

        sendAndReceiveMessages(cf, destination);
        print("Finished (HornetQ client)");
    }

    @Test
    public void testArtemisClient() throws Exception {
        Map<String, Object> transportParams = new HashMap<String, Object>();
        transportParams.put(org.apache.activemq.artemis.core.remoting.impl.netty.TransportConstants.HTTP_UPGRADE_ENABLED_PROP_NAME, true);
        transportParams.put(org.apache.activemq.artemis.core.remoting.impl.netty.TransportConstants.HTTP_UPGRADE_ENDPOINT_PROP_NAME, "http-acceptor");
        transportParams.put(org.apache.activemq.artemis.core.remoting.impl.netty.TransportConstants.PORT_PROP_NAME, 8080);
        org.apache.activemq.artemis.api.core.TransportConfiguration transportConfiguration = new org.apache.activemq.artemis.api.core.TransportConfiguration(org.apache.activemq.artemis.core.remoting.impl.netty.NettyConnectorFactory.class.getName(), transportParams);

        print("Starting to send/receive using Artemis client API");
        ConnectionFactory cf = org.apache.activemq.artemis.api.jms.ActiveMQJMSClient.createConnectionFactoryWithoutHA(org.apache.activemq.artemis.api.jms.JMSFactoryType.CF, transportConfiguration);
        Destination destination = org.apache.activemq.artemis.api.jms.ActiveMQJMSClient.createQueue(QUEUE);

        sendAndReceiveMessages(cf, destination);
        print("Finished (Artemis client)");
    }

    private void print(String message) {
        _logger.info(String.format("[\t%s]: %s", (System.currentTimeMillis() - start), message));
    }
}
